@@ -369,13 +369,14 @@ void ChatView::parseUrls(QString &text) const
   while ( wwwPosition >= 0 )
   {
     httpText = text.mid( wwwPosition - 7, 7 );
-    if ( httpText != "http://" )
+    if ( httpText != "http://" && httpText != "ttps://" )
     {
       text.insert( wwwPosition, "http://" );
     }
     wwwPosition = text.find("www.", wwwPosition + 11);
   }
   // Then parse for common expressions.
+  parseUrlsForTag( text, "https:" );
   parseUrlsForTag( text, "http:" );
   parseUrlsForTag( text, "ftp:" );
   parseUrlsForTag( text, "mailto:" );
