@@ -1501,15 +1501,68 @@ void KMessView::ToolTip::maybeTip(const QPoint &cursorPosition)
   QString        tipText;
   QString        name;
   QString        handle;
+  QString        status, statusname;
+  QString        iconPath;
 
   if(contact != 0)
   {
     name   = QStyleSheet::escape(contact->getTrueFriendlyName());
     handle = QStyleSheet::escape(contact->getHandle());
-    // TODO: Add status beteen 'name' and 'handle'
+    status  = QStyleSheet::escape(contact->getStatus());
+    iconPath = contact->getContactPicturePath();
+
+    // convert AWY to Away etc.
+    if ( status == "AWY" )
+    {
+      statusname = i18n( "(Away)" );
+    }
+    else if ( status == "BRB" )
+    {
+      statusname = i18n( "(Be Right Back)" );
+    }
+    else if ( status == "BSY" )
+    {
+     statusname = i18n( "(Busy)" );
+    }
+    else if ( status == "FLN" )
+    {
+      statusname = "";
+    }
+    else if ( status == "HDN" )
+    {
+      statusname = i18n("(Invisible)");
+    }
+    else if ( status == "IDL" )
+    {
+      statusname = i18n( "(Away - Idle)" );
+    }
+    else if ( status == "LUN" )
+    {
+      statusname = i18n( "(Out to Lunch)" );
+    }
+    else if ( status == "NLN" )
+    {
+      statusname = i18n( "(Online)" );
+    }
+    else if ( status == "PHN" )
+    {
+      statusname = i18n( "(On the Phone)" );
+    }
+    else
+    {
+      statusname = "";
+    }
+
+    tipText  = "<qt>";
+    if( ! iconPath.isEmpty() )
+    {
+      tipText += "<img src=\"" + iconPath + "\" align=left>";
+    }
+    tipText += "&nbsp;<b>" + name + "</b>"  + " " + statusname + "<br>";
+    tipText += handle;
+    tipText += "</qt>";
+
 
-    tipText = "<qt><b>" + name + "</b>"
-            + "<p>" + handle + "</qt>";
     tip( kmessView_->contactListView_->itemRect(currentItem), tipText );
   }
 }
