@@ -285,6 +285,12 @@ void FileTransferP2P::contactStarted1_ContactInvitesUser(const MimeMessage &mess
                                               "Using the switchboard for file transfer (this could be slow).") );
     }
   }
+  else if( contentType == "application/x-msnmsgr-transrespbody" )
+  {
+    kdWarning() << "FileTransferP2P: ignoring unsupported request to make a direct connection." << endl;
+    sendCancelMessage( CANCEL_INVALID_SLP_CONTENT_TYPE );
+    return;
+  }
   else
   {
     kdWarning() << "FileTransferP2P: Received unexpected Content-Type: " << contentType << "." << endl;
