@@ -383,21 +383,65 @@ void P2PApplication::gotAck(const P2PMessage &message)
   }
 
 
+
+  // Are we waiting for the ACK to confirm that all data was received?
+  if(waitingState_ == P2P_WAIT_FOR_DATA_ACK)
+  {
+    waitingState_ = P2P_WAIT_DEFAULT;
+
+#ifdef KMESSDEBUG_P2PAPPLICATION_GENERAL
+    kdDebug() << "P2PApplication: Got an ACK message (all-received ACK)." << endl;
+#endif
+
+    // This should happen after all file/picture data was sent.
+
+
+    if(isUserStartedApp())
+    {
+#ifdef KMESSDEBUG_P2PAPPLICATION_GENERAL
+      kdDebug() << "P2PApplication: All data sent and confirmed, sending BYE message..." << endl;
+#endif
+
+      // We sent the INVITE, now we send the BYE.
+      waitingState_ = P2P_WAIT_DEFAULT;
+      sendSlpBye();
+
+// Disabled in backport
+//      // All data was acked
+//      // Telling derived class the transfer is complete.
+//      showTransferComplete();
+
+      // Don't terminate yet, that message needs to be ACK-ed as well.
+    }
+    else
+    {
+#ifdef KMESSDEBUG_P2PAPPLICATION_GENERAL
+      kdDebug() << "P2PApplication: All data sent and confirmed, waiting for BYE message..." << endl;
+#endif
+
+// Disabled in backport
+//      // All data was acked, this is good enough.
+//      // The remaining teardown happens behind the scenes.
+//      showTransferComplete();
+
+      // The other contact sent the INVITE, so we also wait for the BYE message.
+      // This happens for picture transfers for example.
+
+      waitingState_ = P2P_WAIT_FOR_SLP_BYE;
+      waitingTimer_->start(30000, true); // 30 sec
+    }
+    return;
+  }
+
+
+
   // Are we waiting for the BYE ACK to send our closing-ACK?
   if(waitingState_ == P2P_WAIT_FOR_SLP_BYE_ACK)
   {
     waitingState_ = P2P_WAIT_DEFAULT;
 
 #ifdef KMESSDEBUG_P2PAPPLICATION_GENERAL
-    kdDebug() << "P2PApplication: Got an ACK message (Last ACK, sending Closing-ACK)." << endl;
-#endif
-
-    // This happens if the BYE message was already send.
-    shouldSendAck_ = true;  // ASSERT in sendP2PAck would fail otherwise
-    sendP2PAck(P2PMessage::MSN_FLAG_CLOSING_ACK);
-
-#ifdef KMESSDEBUG_P2PAPPLICATION_GENERAL
-    kdDebug() << "P2PApplication: Terminating session." << endl;
+    kdDebug() << "P2PApplication: Got an ACK message (BYE ACK, terminating)." << endl;
 #endif
 
     // Transfer complete.
@@ -406,21 +450,6 @@ void P2PApplication::gotAck(const P2PMessage &message)
   }
 
 
-  // Are we waiting for the final closing ACK?
-  if(waitingState_ == P2P_WAIT_FOR_CLOSING_ACK)
-  {
-    // HACK: This code is added to support broken clients, the flag field should be 0x40 instead. (not 0x02)
-
-    waitingState_ = P2P_WAIT_DEFAULT;
-
-#ifdef KMESSDEBUG_P2PAPPLICATION_GENERAL
-    kdDebug() << "P2PApplication: Got an ACK message (Closing ACK, terminating application)." << endl;
-#endif
-
-    endApplication();
-    return;
-  }
-
 
   // Are we waiting for the SLP Error message to be ACK-ed?
   if(waitingState_ == P2P_WAIT_FOR_SLP_ERR_ACK)
@@ -793,14 +822,28 @@ void P2PApplication::gotMessage(const MimeMessage &message)
     {
       gotDataPreparation(p2pMessage);
     }
-    // Check for data messages
-    else if(p2pMessage.isPictureData() ||
-            p2pMessage.isFileData()    ||
-            p2pMessage.isFragment())
+    else if( waitingState_ != P2P_WAIT_FOR_FILE_DATA
+         &&  p2pMessage.isPictureData()
+         &&  p2pMessage.getDataSize()  == 4
+         &&  p2pMessage.getTotalSize() == 4 )
     {
-      // HACK: The last p2pMessage.isFragment() check is to
-      // support broken Kopete 0.9.2 code. (has no flag set)
-
+      // HACK: added for Encarta Instant Answers (has data flag set with preparation message).
+      kdWarning() << "P2PApplication: Expecting data-preparation message, got message of 4 bytes with incorrect flags (assuming data preparation, contact=" << contactHandle_ << ")!" << endl;
+      gotDataPreparation(p2pMessage);
+    }
+    // Check for data messages
+    else if(p2pMessage.isPictureData()
+         || p2pMessage.isFileData()
+         || p2pMessage.isFragment())
+    {
+      gotDataFragment(p2pMessage);
+    }
+    else if(waitingState_ == P2P_WAIT_FOR_FILE_DATA
+         && p2pMessage.getFlags() == 0
+         && p2pMessage.isFragment())
+    {
+      // HACK: added for Kopete 0.9.2 code (has no flag set with data messages).
+      kdWarning() << "P2PApplication: Expecting data message, got fragmented message with no flags set (assuming file data, contact=" << contactHandle_ << ")!" << endl;
       gotDataFragment(p2pMessage);
     }
     else
@@ -996,6 +1039,17 @@ void P2PApplication::gotSlpBye(const MimeMessage &/*slpMimeMessage*/)
     return;
   }
 
+  // If the BYE was received early, it may be an indication we
+  // didn't sent something and the other client decided to quit.
+  //
+  // There appears to be a small bug in MSN 7.0. It doesn't sent the final "all data received" ACK,
+  // but sends the BYE directly. After we ACK it, we get a "BYE sent" message back.
+  // This issue appears to be fixed in MSN 7.5.
+  if( waitingState_ != P2P_WAIT_FOR_SLP_BYE && waitingState_ != P2P_WAIT_FOR_DATA_ACK )
+  {
+    kdWarning() << "P2PApplication: Unexpectedly received a SLP BYE message (contact=" << contactHandle_ << ")." << endl;
+  }
+
 
   if(shouldSendAck_)
   {
@@ -1003,27 +1057,15 @@ void P2PApplication::gotSlpBye(const MimeMessage &/*slpMimeMessage*/)
   }
 
 #ifdef KMESSTEST
-  ASSERT( waitingState_ == P2P_WAIT_FOR_SLP_BYE );
+  ASSERT( waitingState_ == P2P_WAIT_FOR_SLP_BYE || waitingState_ == P2P_WAIT_FOR_DATA_ACK );
 #endif
 
-  if(sentCancelMessage_)
-  {
-    // MSN 7 doesn't seam to send the "closing ACK" when we sent an MSNSLP error
-#ifdef KMESSDEBUG_P2PAPPLICATION_GENERAL
-    kdDebug() << "P2PApplication: Got SLP BYE message, not expecting a Closing-ACK since we aborted." << endl;
-#endif
-    endApplication();
-    return;
-  }
-
-
 #ifdef KMESSDEBUG_P2PAPPLICATION_GENERAL
-  kdDebug() << "P2PApplication: Got SLP BYE message, waiting for last Closing-ACK..." << endl;
+  kdDebug() << "P2PApplication: Got SLP BYE message, closing application." << endl;
 #endif
 
-  // We've ACK-ed the message, now we wait for the final "Closing-ACK"
-  waitingState_ = P2P_WAIT_FOR_CLOSING_ACK;
-  waitingTimer_->start(30000, true); // 30 sec
+  endApplication();
+  return;
 }
 
 
@@ -1079,7 +1121,8 @@ void P2PApplication::gotSlpInvite(const MimeMessage &slpMimeMessage)
 
   // Validate the content type
   if(invitationContentType_ != "application/x-msnmsgr-sessionreqbody" &&
-     invitationContentType_ != "application/x-msnmsgr-transreqbody")
+     invitationContentType_ != "application/x-msnmsgr-transreqbody" &&   // HACK: added for KMess 1.4.2 (sent incorrect value)
+     invitationContentType_ != "application/x-msnmsgr-transrespbody" )
   {
     kdWarning() << "P2PApplication: Received unexpected Content-Type: " << invitationContentType_ << "." << endl;
 
@@ -2010,7 +2053,10 @@ void P2PApplication::sendSlpOkMessage(const MimeMessage &message)
   kdDebug() << "P2PApplication: Sending MSNSLP/1.0 200 OK message." << endl;
 #endif
 
-  QString content = message.getFields();
+  // Get message content and Content-Type
+  bool gotTransferInvitation = (invitationContentType_ == "application/x-msnmsgr-transreqbody");
+  QString content     = message.getFields();
+  QString contentType = (gotTransferInvitation ? "application/x-msnmsgr-transrespbody" : invitationContentType_);
 
   QString myHandle = CurrentAccount::instance()->getHandle();
 
@@ -2023,7 +2069,7 @@ void P2PApplication::sendSlpOkMessage(const MimeMessage &message)
                "CSeq: " + QString::number(invitationCSeq_ + 1) + "\r\n"
                "Call-ID: " + callID_ + "\r\n"
                "Max-Forwards: 0\r\n"
-               "Content-Type: " + invitationContentType_ + "\r\n"
+               "Content-Type: " + contentType + "\r\n"
                "Content-Length: " + QString::number(content.length() + 3) + "\r\n"
                "\r\n"
                + content + "\r\n";
@@ -2160,7 +2206,7 @@ void P2PApplication::slotSendData()
   data.duplicate( buffer, bytesRead );
 
 #ifdef KMESSDEBUG_P2PAPPLICATION_GENERAL
-  kdDebug() << "P2PApplication: sending data."
+  kdDebug() << "P2PApplication: Sending data."
             << " offset = " << messageOffset_
             << " buffer = " << bytesRead
             << endl;
@@ -2184,33 +2230,34 @@ void P2PApplication::slotSendData()
     sendP2PMessage(data, 0, 1); // The best solution in this situation
   }
 
+// Disabled in backport
+//  // Dispatch the progress to the derived class
+//  showTransferProgress( messageOffset_ + bytesRead );
 
   // Finished?
   if(messageTotalSize_ > 0)
   {
     // Run this slot again after 10 ms.
     QTimer::singleShot( 10, this, SLOT(slotSendData()) );
-    return;
   }
-
-
-  // Finished!
+  else
+  {
+    // Transfer finished!
 #ifdef KMESSTEST
-  ASSERT( dataSource_->atEnd() );
+    ASSERT( dataSource_->atEnd() );
 #endif
+
+    // Reset data source
+    dataSource_   = 0;
+    dataType_     = P2P_TYPE_NEGOTIATION;
+
 #ifdef KMESSDEBUG_P2PAPPLICATION_GENERAL
-  kdDebug() << "P2PApplication: Finished sending data, waiting for BYE message..." << endl;
+    kdDebug() << "P2PApplication: Finished sending data, waiting for ACK..." << endl;
 #endif
 
-  dataSource_   = 0;
-  dataType_     = P2P_TYPE_NEGOTIATION;
-
-  // We transmitted our file, the caller could
-  // close the session now by sending a BYE message.
-
-
-  waitingState_ = P2P_WAIT_FOR_SLP_BYE;
-  waitingTimer_->start(30000, true); // 30 sec
+    waitingState_ = P2P_WAIT_FOR_DATA_ACK;
+    waitingTimer_->start(30000, true); // 30 sec
+  }
 }
 
 
