@@ -24,6 +24,7 @@
 #include <kdebug.h>
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <kurl.h>
 
 #include "../currentaccount.h"
 #include "../kmessdebug.h"
@@ -90,6 +91,8 @@ MsnConnection::~MsnConnection()
 // Insert "%" values for certain regular expressions, i.e. "%20" for " ".
 void MsnConnection::addPercents(QString &word) const
 {
+  // TODO: replace with KURL::encode_string(), but test for compatibility with other clients.
+  // Currently left alone, might cause more problems and needs more testing.
   word = word.replace(QRegExp(" "),"%20");
 }
 
@@ -461,101 +464,7 @@ bool MsnConnection::isConnected() const
 // Replace any "%" values with their regular values, i.e. "%20" to " ".
 void MsnConnection::removePercents(QString &word) const
 {
-  word = word.replace( QRegExp( "%20" ), " " );
-  word = word.replace( QRegExp( "%21" ), "!" );
-  word = word.replace( QRegExp( "%22" ), "\"" );
-  word = word.replace( QRegExp( "%23" ), "#" );
-  word = word.replace( QRegExp( "%24" ), "$" );
-  word = word.replace( QRegExp( "%25" ), "%" );
-  word = word.replace( QRegExp( "%26" ), "&" );
-  word = word.replace( QRegExp( "%27" ), "'" );
-  word = word.replace( QRegExp( "%28" ), "(" );
-  word = word.replace( QRegExp( "%29" ), ")" );
-  word = word.replace( QRegExp( "%2A" ), "*" );
-  word = word.replace( QRegExp( "%2B" ), "+" );
-  word = word.replace( QRegExp( "%2C" ), "," );
-  word = word.replace( QRegExp( "%2D" ), "-" );
-  word = word.replace( QRegExp( "%2E" ), "." );
-  word = word.replace( QRegExp( "%2F" ), "/" );
-  word = word.replace( QRegExp( "%30" ), "0" );
-  word = word.replace( QRegExp( "%31" ), "1" );
-  word = word.replace( QRegExp( "%32" ), "2" );
-  word = word.replace( QRegExp( "%33" ), "3" );
-  word = word.replace( QRegExp( "%34" ), "4" );
-  word = word.replace( QRegExp( "%35" ), "5" );
-  word = word.replace( QRegExp( "%36" ), "6" );
-  word = word.replace( QRegExp( "%37" ), "7" );
-  word = word.replace( QRegExp( "%38" ), "8" );
-  word = word.replace( QRegExp( "%39" ), "9" );
-  word = word.replace( QRegExp( "%3A" ), ":" );
-  word = word.replace( QRegExp( "%3B" ), ";" );
-  word = word.replace( QRegExp( "%3C" ), "<" );
-  word = word.replace( QRegExp( "%3D" ), "=" );
-  word = word.replace( QRegExp( "%3E" ), ">" );
-  word = word.replace( QRegExp( "%3F" ), "?" );
-  word = word.replace( QRegExp( "%40" ), "@" );
-  word = word.replace( QRegExp( "%41" ), "A" );
-  word = word.replace( QRegExp( "%42" ), "B" );
-  word = word.replace( QRegExp( "%43" ), "C" );
-  word = word.replace( QRegExp( "%44" ), "D" );
-  word = word.replace( QRegExp( "%45" ), "E" );
-  word = word.replace( QRegExp( "%46" ), "F" );
-  word = word.replace( QRegExp( "%47" ), "G" );
-  word = word.replace( QRegExp( "%48" ), "H" );
-  word = word.replace( QRegExp( "%49" ), "I" );
-  word = word.replace( QRegExp( "%4A" ), "J" );
-  word = word.replace( QRegExp( "%4B" ), "K" );
-  word = word.replace( QRegExp( "%4C" ), "L" );
-  word = word.replace( QRegExp( "%4D" ), "M" );
-  word = word.replace( QRegExp( "%4E" ), "N" );
-  word = word.replace( QRegExp( "%4F" ), "O" );
-  word = word.replace( QRegExp( "%50" ), "P" );
-  word = word.replace( QRegExp( "%51" ), "Q" );
-  word = word.replace( QRegExp( "%52" ), "R" );
-  word = word.replace( QRegExp( "%53" ), "S" );
-  word = word.replace( QRegExp( "%54" ), "T" );
-  word = word.replace( QRegExp( "%55" ), "U" );
-  word = word.replace( QRegExp( "%56" ), "V" );
-  word = word.replace( QRegExp( "%57" ), "W" );
-  word = word.replace( QRegExp( "%58" ), "X" );
-  word = word.replace( QRegExp( "%59" ), "Y" );
-  word = word.replace( QRegExp( "%5A" ), "Z" );
-  word = word.replace( QRegExp( "%5B" ), "[" );
-  word = word.replace( QRegExp( "%5C" ), "\\" );
-  word = word.replace( QRegExp( "%5D" ), "]" );
-  word = word.replace( QRegExp( "%5E" ), "^" );
-  word = word.replace( QRegExp( "%5F" ), "_" );
-  word = word.replace( QRegExp( "%60" ), "`" );
-  word = word.replace( QRegExp( "%61" ), "a" );
-  word = word.replace( QRegExp( "%62" ), "b" );
-  word = word.replace( QRegExp( "%63" ), "c" );
-  word = word.replace( QRegExp( "%64" ), "d" );
-  word = word.replace( QRegExp( "%65" ), "e" );
-  word = word.replace( QRegExp( "%66" ), "f" );
-  word = word.replace( QRegExp( "%67" ), "g" );
-  word = word.replace( QRegExp( "%68" ), "h" );
-  word = word.replace( QRegExp( "%69" ), "i" );
-  word = word.replace( QRegExp( "%6A" ), "j" );
-  word = word.replace( QRegExp( "%6B" ), "k" );
-  word = word.replace( QRegExp( "%6C" ), "l" );
-  word = word.replace( QRegExp( "%6D" ), "m" );
-  word = word.replace( QRegExp( "%6E" ), "n" );
-  word = word.replace( QRegExp( "%6F" ), "o" );
-  word = word.replace( QRegExp( "%70" ), "p" );
-  word = word.replace( QRegExp( "%71" ), "q" );
-  word = word.replace( QRegExp( "%72" ), "r" );
-  word = word.replace( QRegExp( "%73" ), "s" );
-  word = word.replace( QRegExp( "%74" ), "t" );
-  word = word.replace( QRegExp( "%75" ), "u" );
-  word = word.replace( QRegExp( "%76" ), "v" );
-  word = word.replace( QRegExp( "%77" ), "w" );
-  word = word.replace( QRegExp( "%78" ), "X" );
-  word = word.replace( QRegExp( "%79" ), "y" );
-  word = word.replace( QRegExp( "%7A" ), "z" );
-  word = word.replace( QRegExp( "%7B" ), "{" );
-  word = word.replace( QRegExp( "%7C" ), "|" );
-  word = word.replace( QRegExp( "%7D" ), "}" );
-  word = word.replace( QRegExp( "%7E" ), "~" );
+  word = KURL::decode_string(word);
 }
 
 
