@@ -300,7 +300,7 @@ SslLoginHandler* MsnNotificationConnection::createLoginHandler()
 void MsnNotificationConnection::dotNetMessage(QString message)
 {
   int    result;
-  KURL   netURL( "http://messenger.msn.com/support/status.aspx" );
+  KURL   netURL( "http://status.messenger.msn.com/Status.aspx" );
 
   message += "\n" + i18n("Show .NET service status?");
   result = KMessageBox::questionYesNo( 0, message, i18n("Server Error") );
