@@ -131,8 +131,10 @@ void MsnSwitchboardConnection::closeConnection()
     kdWarning() << "Switchboard::closeConnection: unable to terminate remaining applications nicely." << endl;
     mimeApplications_.setAutoDelete(true);
     mimeApplications_.clear();
+    mimeApplications_.setAutoDelete(false);  // and reset again!
     p2pApplications_.setAutoDelete(true);
     p2pApplications_.clear();
+    p2pApplications_.setAutoDelete(false);  // and reset again!
   }
 
   // Detach contacts, they can't use this switchboard anymore
@@ -1393,6 +1395,7 @@ void MsnSwitchboardConnection::parseP2PAppMessage(const QString &sendingContact,
             << " AckID="       << message.getAckMessageID()
             << " ACKUID="      << message.getAckUniqueID()
             << " flags="       << message.getFlags()
+            << " size="        << message.getDataSize()
             << " no.sessions=" << p2pApplications_.count() << endl;
 #endif
 
@@ -1416,7 +1419,8 @@ void MsnSwitchboardConnection::parseP2PAppMessage(const QString &sendingContact,
     // Session ID is 0 if clients negotiate.
     // (unfortunately for both INVITE and BYE messages!)
 
-    if( message.getFlags() == 0 )
+    if( message.getFlags() == 0
+     && message.getDataSize() > 0)  // HACK: added for Bot2K3 4.1 (has no flags with first ack message).
     {
       if( message.getDataOffset() > 0 )
       {
@@ -1432,7 +1436,7 @@ void MsnSwitchboardConnection::parseP2PAppMessage(const QString &sendingContact,
 
           if(p2pIterator.current()->getLastContactMessageID() == messageID)
           {
-            // HACK: Not tested for "getLastContactAckMessageID() == ackMessageID" to support broken implementations.
+            // HACK: Not tested for "&& getLastContactAckMessageID() == ackMessageID" to support non-standard implementations.
 
             // Found it, save reference
             p2pApplication = p2pIterator.current();
@@ -1447,7 +1451,7 @@ void MsnSwitchboardConnection::parseP2PAppMessage(const QString &sendingContact,
       {
         // No offset, start of SLP message
 
-        // FIXME: Assuming that the Call-ID will be found in the first part..
+        // NOTE: Assuming that the Call-ID will be found in the first part..
         slpMessage = QString::fromUtf8( message.getData(), message.getDataSize() );
         callPos    = callRE.search(slpMessage);
 
@@ -1507,14 +1511,21 @@ void MsnSwitchboardConnection::parseP2PAppMessage(const QString &sendingContact,
 #endif
 
         if(  p2pIterator.current()->getLastUserUniqueID()  == ackUniqueID   // This works with MSN 6.2
-        ||  (p2pIterator.current()->getLastUserUniqueID()  == ackMessageID && ackUniqueID == 0)  // For type-8 error messages
-        ||   p2pIterator.current()->getLastUserMessageID() == ackMessageID) // HACK: added to support broken GAIM 1.1.0 code
+        ||  (p2pIterator.current()->getLastUserUniqueID()  == ackMessageID && ackUniqueID == 0) )  // For type-8 error messages
         {
           // Found it, save reference
           p2pApplication = p2pIterator.current();
           break;
         }
 
+        if( ackUniqueID == 0 && p2pIterator.current()->getLastUserMessageID() == ackMessageID )
+        {
+          // HACK: added for GAIM 1.5 (does not return UniqueID)
+          kdWarning() << "SwitchBoard: Got P2P message with matching ackMessageID and no ackUniqueID set (delivering anyway, contact=" << sendingContact << ")!" << endl;
+          p2pApplication = p2pIterator.current();
+          break;
+        }
+
         ++p2pIterator;
       }
 
@@ -1522,15 +1533,22 @@ void MsnSwitchboardConnection::parseP2PAppMessage(const QString &sendingContact,
       if(p2pApplication == 0)
       {
         kdWarning() << "SwitchBoard: P2P message can't be handled, "
-                    << "ACK-Message-ID not found (flags=" << message.getFlags() << " contact=" << sendingContact << ")." << endl;
+                    << "ACK-Message-ID not found (flags=" << message.getFlags() << " size=" << message.getDataSize() << " contact=" << sendingContact << ")." << endl;
         return;
       }
     }
+    else if( message.getFlags() == 0 && message.getDataSize() == 0 && ackUniqueID == 0 )
+    {
+      // HACK: added bot Bot2k3 4.1 (sends first ack without UniqueID, flags or any other context).
+      kdWarning() << "SwitchBoard P2P message has no ACK fields set at all (assuming base identifier message, contact=" << sendingContact << ")!" << endl;
+      return;
+    }
     else
     {
       // Didn't reconize the flag type.
       kdWarning() << "SwitchBoard: P2P message can't be handled, "
-                  << "unknown flag type found (flags=" << message.getFlags() << " size="  << message.getDataSize() << ")." << endl;
+                  << "unknown flag type found (flags=" << message.getFlags() << " size="  << message.getDataSize() << " contact=" << sendingContact << ")." << endl;
+
     }
   }
   else
@@ -1558,7 +1576,7 @@ void MsnSwitchboardConnection::parseP2PAppMessage(const QString &sendingContact,
     if(p2pApplication == 0)
     {
       kdWarning() << "SwitchBoard: Unable to handle P2P message, "
-                  << "Session-ID not found (contact=" << sendingContact << ")." << endl;
+                  << "Session-ID not found (flags="   << message.getFlags() << " size=" << message.getDataSize() << " contact=" << sendingContact << ")." << endl;
     }
   }
 
@@ -1926,19 +1944,25 @@ void MsnSwitchboardConnection::startChat(ChatInformation *chatInfo)
       return;
     }
 
-    if(contactsInChat_.isEmpty() || isExclusiveChatWithContact(chatInfo->getContactHandle()))
-    {
-#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
-      kdDebug() << "Switchboard - Contact resumed the chat but uses a different server, reconnecting." << endl;
-#endif
-      closeConnection();
-    }
-    else
+    if( ! contactsInChat_.isEmpty() && ! isExclusiveChatWithContact(chatInfo->getContactHandle()))
     {
       kdWarning() << "Switchboard is already connected to '" << chatInfo_->getIp() << ":" << chatInfo_->getPort() << "', "
                   << "cant't start new chat with '" << chatInfo_->getContactHandle() << "'!" << endl;
       return;
     }
+
+#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
+    kdDebug() << "Switchboard - Contact resumed the chat but uses a different server, reconnecting." << endl;
+#endif
+    closeConnection();
+  }
+
+  if( p2pApplications_.count() > 0 || mimeApplications_.count() > 0 )
+  {
+#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
+    kdDebug() << "Switchboard: re-connecting but there are still p2p applications open, terminating applications now." << endl;
+#endif
+    closeConnection();
   }
 
 #ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
