@@ -213,7 +213,7 @@ void SettingsDialog::contactFontClicked()
   bool    accepted;
 
   // Get a default font from the font button
-  fontFamily = chattingWidget_->contactFontButton_->text();
+  fontFamily = chattingWidget_->contactFontButton_->text().replace("&",QString::null);
   font = chattingWidget_->contactFontButton_->font();
   // Get a new font and font family from the user
   accepted = getFont( font, fontFamily );
@@ -235,7 +235,7 @@ void SettingsDialog::fontClicked()
   bool    accepted;
 
   // Get a default font from the font button
-  fontFamily = chattingWidget_->fontButton_->text();
+  fontFamily = chattingWidget_->fontButton_->text().replace("&",QString::null);
   font = chattingWidget_->fontButton_->font();
   // Get a new font and font family from the user
   accepted = getFont( font, fontFamily );
